@@ -19,7 +19,7 @@
 /*
  * Configuration backend.
  *
- * Copyright (C) 2003-2005 Isak Savo
+ * Copyright (C) 2003-2006 Isak Savo
  */
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
@@ -55,7 +55,8 @@ gboolean cfg_write_settings (void)
 		return FALSE;
 	}
     
-	fputs ("# ~/.pureadminrc - Configuration file for pureadmin\n"
+	fputs ("# -*- sh -*-\n"
+	       "# ~/.pureadminrc - Configuration file for pureadmin\n"
 	       "# All variables are in the form VARNAME=VALUE where VALUE should be an UN-quoted string\n"
 	       "# Comment lines BEGINS with a '#'. Otherwise the # is considered a part of the value/variablename\n\n", f);
 
@@ -65,18 +66,29 @@ gboolean cfg_write_settings (void)
 	if (cfg.default_gid > 0)
 		fprintf (f, "defaultgid=%d\n\n", cfg.default_gid);
   
-	fputs ("# Default home is the directory which will be assigned to new users by default.\n", f);
-	fprintf (f, "defaulthome=%s\n\n", cfg.default_home);
+	if (cfg.default_home)
+	{
+		fputs ("# Default home is the directory which will be assigned to new users by default.\n", f);
+		fprintf (f, "defaulthome=%s\n\n", cfg.default_home);
+	}
   
-	fputs ("# Name and optional full path to the pure-pw command. Path is only needed if it is not installed in a standard location (i.e. Not in $PATH)\n", f);
-	fprintf (f, "cmd_purepw=%s\n\n", cfg.cmd_purepw);
+	if (cfg.cmd_purepw)
+	{
+		fputs ("# Name and optional full path to the pure-pw command. Path is only needed if it is not installed in a standard location (i.e. Not in $PATH)\n", f);
+		fprintf (f, "cmd_purepw=%s\n\n", cfg.cmd_purepw);
+	}
   
 	fputs ("# Name and location of the password files where pureftpd reads user-information. The pdb-file is a binary database file,\n", f);
-	fprintf (f, "pwfile=%s\n", cfg.pwfile);
-	fprintf (f, "pdbfile=%s\n\n", cfg.pdbfile);
+	if (cfg.pwfile)
+		fprintf (f, "pwfile=%s\n", cfg.pwfile);
+	if (cfg.pdbfile)
+		fprintf (f, "pdbfile=%s\n\n", cfg.pdbfile);
   
-	fputs ("# Location of the logfile where pureftpd writes logging info\n", f);
-	fprintf (f, "logfile=%s\n\n", cfg.logfile);
+	if (cfg.logfile)
+	{
+		fputs ("# Location of the logfile where pureftpd writes logging info\n", f);
+		fprintf (f, "logfile=%s\n\n", cfg.logfile);
+	}
 	fputs ("# Logging method, 0 means syslog and 1 means custom logfile\n", f);
 	fprintf (f, "logmethod=%d\n", cfg.logmethod);
 
@@ -104,6 +116,7 @@ gboolean cfg_write_settings (void)
 	fprintf (f, "window_size=%d:%d\n", cfg.win_size[0], cfg.win_size[1]);
 	fprintf (f, "window_pos=%d:%d\n", cfg.win_pos[0], cfg.win_pos[1]);
 	fprintf (f, "divider_pos=%d\n", cfg.div_pos);
+	fprintf (f, "show_advinfo=%d\n", cfg.show_advinfo);
 
 	fclose (f);
 	return TRUE;
@@ -140,67 +153,87 @@ gboolean cfg_read_settings (void)
 			continue;
 		g_strstrip (arr[0]);
 		g_strstrip (arr[1]);
+		gchar *key = arr[0];
+		gchar *val = arr[1];
       
-		if (!*arr[1]) /* Ignore values that is empty (i.e. "") */
+		if (*val == '\0') /* Ignore values that is empty (i.e. "") */
 			continue;
-		if (strncmp (arr[0], "defaultuid", 10) == 0)
-			cfg.default_uid = atoi (arr[1]);
-		else if (strncmp (arr[0], "defaultgid", 10) == 0)
-			cfg.default_gid = atoi (arr[1]);
-		else if (strncmp (arr[0], "defaulthome", 11) == 0)
+		if (strncmp (key, "defaultuid", 10) == 0)
+			cfg.default_uid = atoi (val);
+		else if (strncmp (key, "defaultgid", 10) == 0)
+			cfg.default_gid = atoi (val);
+		else if (strncmp (key, "defaulthome", 11) == 0)
 		{
-			g_free (cfg.default_home);
-			cfg.default_home = g_strdup (arr[1]);
+			if (g_path_is_absolute(val))
+			{
+				g_free (cfg.default_home);
+				cfg.default_home = g_strdup (val);
+			}
 		}
-		else if (strncmp (arr[0], "cmd_purepw", 10) == 0)
+		else if (strncmp (key, "cmd_purepw", 10) == 0)
 		{
-			g_free (cfg.cmd_purepw);
-			cfg.cmd_purepw = g_strdup (arr[1]);
+			if ((g_path_is_absolute(val) && g_file_test(val, G_FILE_TEST_EXISTS)) ||
+				g_find_program_in_path(val))
+			{
+				g_free (cfg.cmd_purepw);
+				cfg.cmd_purepw = g_strdup (val);
+			}
 		}
-		else if (strncmp (arr[0], "pwfile", 6) == 0)
+		else if (strncmp (key, "pwfile", 6) == 0)
 		{
-			g_free (cfg.pwfile);
-			cfg.pwfile = g_strdup (arr[1]);
+			if (g_path_is_absolute(val))
+			{
+				g_free (cfg.pwfile);
+				cfg.pwfile = g_strdup (val);
+			}
 		}
-		else if (strncmp (arr[0], "pdbfile", 7) == 0)
+		else if (strncmp (key, "pdbfile", 7) == 0)
 		{
-			g_free (cfg.pdbfile);
-			cfg.pdbfile = g_strdup (arr[1]);
+			if (g_path_is_absolute(val))
+			{
+				g_free (cfg.pdbfile);
+				cfg.pdbfile = g_strdup (val);
+			}
 		}
-		else if (strncmp (arr[0], "logfile", 7) == 0)
+		else if (strncmp (key, "logfile", 7) == 0)
 		{
-			g_free (cfg.logfile);
-			cfg.logfile = g_strdup (arr[1]);
+			if (g_path_is_absolute(val))
+			{
+				g_free (cfg.logfile);
+				cfg.logfile = g_strdup (val);
+			}
 		}
-		else if (strncmp (arr[0], "logmethod", 9) == 0)
+		else if (strncmp (key, "logmethod", 9) == 0)
 		{
-			cfg.logmethod = atoi (arr[1]);
+			cfg.logmethod = atoi (val);
 			if (cfg.logmethod < 0 || cfg.logmethod >= NUM_LOGMETHODS)
 				cfg.logmethod = 0;
 		}
-		else if (strncmp (arr[0], "window_size", 11) == 0)
-			sscanf (arr[1],"%d:%d",&(cfg.win_size[0]), &(cfg.win_size[1]));
-		else if (strncmp (arr[0], "window_pos", 10) == 0)
-			sscanf (arr[1],"%d:%d",&(cfg.win_pos[0]), &(cfg.win_pos[1]));
-		else if (strncmp (arr[0], "divider_pos", 11) == 0)
-			cfg.div_pos = atoi (arr[1]);
-		else if (strncmp (arr[0], "save_window_geometry", 20) == 0)
-			cfg.save_window_geometry = atoi (arr[1]);
-		else if (strncmp (arr[0], "resolve_hostnames", 17) == 0)
-			cfg.resolve_hostnames = atoi (arr[1]);
-		else if (strncmp (arr[0], "seen_usrdlg_welcome", 19) == 0)
-			cfg.seen_usrdlg_welcome = atoi (arr[1]);
-		else if (strncmp (arr[0], "show_fam_errmsg", 15) == 0)
-			cfg.show_fam_errmsg = atoi (arr[1]);
-		else if (strncmp (arr[0], "use_tray_icon", 13) == 0)
-			cfg.use_tray_icon = atoi (arr[1]);
-		else if (strncmp (arr[0], "username_encoding", 17) == 0)
+		else if (strncmp (key, "window_size", 11) == 0)
+			sscanf (val,"%d:%d",&(cfg.win_size[0]), &(cfg.win_size[1]));
+		else if (strncmp (key, "window_pos", 10) == 0)
+			sscanf (val,"%d:%d",&(cfg.win_pos[0]), &(cfg.win_pos[1]));
+		else if (strncmp (key, "divider_pos", 11) == 0)
+			cfg.div_pos = atoi (val);
+		else if (strncmp (key, "save_window_geometry", 20) == 0)
+			cfg.save_window_geometry = atoi (val);
+		else if (strncmp (key, "show_advinfo", 12) == 0)
+			cfg.show_advinfo = atoi (val);
+		else if (strncmp (key, "resolve_hostnames", 17) == 0)
+			cfg.resolve_hostnames = atoi (val);
+		else if (strncmp (key, "seen_usrdlg_welcome", 19) == 0)
+			cfg.seen_usrdlg_welcome = atoi (val);
+		else if (strncmp (key, "show_fam_errmsg", 15) == 0)
+			cfg.show_fam_errmsg = atoi (val);
+		else if (strncmp (key, "use_tray_icon", 13) == 0)
+			cfg.use_tray_icon = atoi (val);
+		else if (strncmp (key, "username_encoding", 17) == 0)
 		{
 			g_free (cfg.uname_encoding);
-			cfg.uname_encoding = g_strdup (arr[1]);
+			cfg.uname_encoding = g_strdup (val);
 		}
-		else if (strncmp (arr[0], "use_system_encoding", 19) == 0)
-			cfg.use_system_encoding = atoi (arr[1]);
+		else if (strncmp (key, "use_system_encoding", 19) == 0)
+			cfg.use_system_encoding = atoi (val);
       
 		g_strfreev (arr);
 	}
@@ -232,8 +265,7 @@ gchar *cfg_find_default_home (void)
 	else if (g_file_test ("/home/ftp", G_FILE_TEST_IS_DIR))
 		return g_strdup ("/home/ftp");
 
-	/* FIXME: Default to ""? */
-	return NULL;
+	return g_strdup ("/home/ftpusers");
 }
 
 gchar *cfg_find_pwfile (void)
@@ -319,6 +351,43 @@ guint cfg_find_ftpgroup_gid (void)
 	return rv;
 }
 
+gchar *cfg_get_pftp_config_dir (void)
+{
+	gchar *dir = NULL;
+	if (cfg.pwfile)
+		dir = g_path_get_dirname (cfg.pwfile);
+	else if (cfg.pdbfile)
+		dir = g_path_get_dirname (cfg.pdbfile);
+	else if (g_file_test("/etc/pure-ftpd", G_FILE_TEST_IS_DIR))
+		dir = g_strdup ("/etc/pure-ftpd");
+	else
+		dir = g_strdup ("/etc");
+
+	return dir;
+}
+
+/* Gets the default password file for pureftpd, regardless of whether it exists or not */
+gchar *cfg_get_pftp_default_pwfile (void)
+{
+	if (g_getenv ("PURE_PASSWDFILE") && g_path_is_absolute(g_getenv("PURE_PASSWDFILE")))
+		return g_strdup (g_getenv ("PURE_PASSWDFILE"));
+	gchar *dir = cfg_get_pftp_config_dir();
+	gchar *f = g_build_filename(dir, "pureftpd.passwd", NULL);
+	g_free(dir);
+	return f;
+}
+
+/* Gets the default dbfile for pureftpd, regardless of whether it exists or not */
+gchar *cfg_get_pftp_default_pdbfile (void)
+{
+	if (g_getenv ("PURE_DBFILE") && g_path_is_absolute(g_getenv("PURE_DBFILE")))
+		return g_strdup (g_getenv ("PURE_DBFILE"));
+	gchar *dir = cfg_get_pftp_config_dir();
+	gchar *f = g_build_filename(dir, "pureftpd.pdb", NULL);
+	g_free(dir);
+	return f;
+}
+
 /* cfg_fill_missing: Tries to find sane defaults for values that there are no
  *		     configured setting for. This is mostly done by probing the
  *		     file system for the relevant files.
@@ -330,19 +399,29 @@ void cfg_fill_missing (void)
 	if (!cfg.cmd_ftpwho)
 		cfg.cmd_ftpwho = g_find_program_in_path ("pure-ftpwho");
 	if (!cfg.cmd_startstop) {
-		gchar prg_arr[3][20] = { "rc.pure-ftpd", "rc.pureftpd", "pure-ftpd" };
-		gchar *s;
+		gchar prg_arr[][20] = { "rc.pure-ftpd", "rc.pureftpd", "pure-ftpd", "pure-ftpd-mysql"};
 
-		s = *prg_arr;
-		while (s){
+		for (gint i = 0; i < G_N_ELEMENTS(prg_arr); i++)
+		{
+			gchar *s = prg_arr[i];
 			cfg.cmd_startstop = misc_find_prog_in_dirs
 				(s, "/etc/rc.d /etc/rc.d/init.d /etc/init.d /usr/local/etc/rc.d");
 			if (cfg.cmd_startstop)
 				break;
-			s++;
 		}
 	}
-
+	if (!cfg.pwfile)
+	{
+		gchar *base = cfg_get_pftp_config_dir();
+		cfg.pwfile = g_build_filename (base, "pureftpd.passwd", NULL);
+		g_free (base);
+	}
+	if (!cfg.pdbfile)
+	{
+		gchar *base = cfg_get_pftp_config_dir();
+		cfg.pdbfile = g_build_filename (base, "pureftpd.pdb", NULL);
+		g_free (base);
+	}
 	if (!cfg.default_home) 
 		cfg.default_home = cfg_find_default_home ();
 	if (!cfg.pwfile)
