@@ -25,10 +25,10 @@
 #ifndef __FAMWRAP_H__
 #define __FAMWRAP_H__
 
-gboolean fam_add_watch (const gchar *fname, void (*func));
-gboolean fam_suspend_watch (const gchar *fname);
-gboolean fam_resume_watch (const gchar *fname);
-gboolean fam_delete_watch (const gchar *fname);
+gboolean fam_set_watch (const gchar *fname, void (*func));
+gboolean fam_suspend_watch (void);
+gboolean fam_resume_watch (void);
+gboolean fam_delete_watch (void);
 gboolean fam_terminate (void);
 gboolean fam_is_running (void);
 
