@@ -44,17 +44,17 @@
 # define G_MAXINT32 ((gint32) 0x7fffffff)
 #endif
 
+extern gboolean debug;
 /* Set _CRITICAL since _ERR will terminate the program by glib */
 #define pur_log_err(...) \
   g_log (NULL, G_LOG_LEVEL_CRITICAL, __VA_ARGS__);
 #define pur_log_wrn(...) \
   g_log (NULL, G_LOG_LEVEL_WARNING, __VA_ARGS__);
-#ifdef ENABLE_DEBUG
-# define pur_log_dbg(...) \
-  g_log (NULL, G_LOG_LEVEL_DEBUG, __VA_ARGS__);
-#else
-# define pur_log_dbg(...)
-#endif /* ENABLE_DEBUG */
+#define pur_log_dbg(...) \
+G_STMT_START { \
+if (debug) \
+ g_log (NULL, G_LOG_LEVEL_DEBUG, __VA_ARGS__); \
+} G_STMT_END 
 #define pur_log_nfo(...) \
   g_log (NULL, G_LOG_LEVEL_INFO, __VA_ARGS__);
 
@@ -84,8 +84,6 @@
 #  define N_(String) (String)
 #endif
 
-void get_srv_vars ();
-
 typedef enum {
 	RUNMODE_STOPPED = 0, /* Server is stopped */
 	RUNMODE_STANDALONE, /* Running in standalone mode */
