@@ -37,6 +37,7 @@
 #include "usr_manager.h"
 #include "cfg.h"
 #include "binreloc.h"
+#include "system_accounts.h"
 
 EggStatusIcon *status_icon = NULL;
 
@@ -53,71 +54,28 @@ static gchar *sec_to_time (gulong sec)
 	retval = g_strdup_printf ("%02d:%02d:%02d", hr, min, (gint) sec);
 	return retval;
 }
-   
-static gchar *number_separate (gint num)
+
+static gchar *get_size_string (guint kbytes)
 {
-	GString *str;
-	gchar *retval;
-	gboolean more_prev = FALSE;
-   
-	str = g_string_new ("");
-	if (num > 1000000000)
-	{
-		g_string_append_printf (str, "%03d,", num / 1000000000);
-		num = num % 1000000000;
-		more_prev = TRUE;
-	}
-	if (num > 1000000)
-	{
-		if (more_prev)
-			g_string_append_printf (str, "%03d,", num / 1000000);
-		else
-			g_string_append_printf (str, "%d,", num / 1000000);
-		num = num % 1000000;
-		more_prev = TRUE;
-	}
-	if (num > 1000)
-	{
-		if (more_prev)
-			g_string_append_printf (str, "%03d,", num / 1000);
-		else
-			g_string_append_printf (str, "%d,", num / 1000);
-		num = num % 1000;
-		more_prev = TRUE;
-	}
-	g_string_append_printf (str, "%d", num);
-	retval = str->str;
-	g_string_free (str, FALSE);
-	return retval;
+	if (kbytes > 1024 * 1024)
+		return g_strdup_printf ("%.2lf %s", (gdouble) kbytes / (1024.0 * 1024.0), _("GiB"));
+	if (kbytes > 1024)
+		return g_strdup_printf ("%.2lf %s", (gdouble) kbytes / 1024.0, _("MiB"));
+
+	return g_strdup_printf ("%d %s", kbytes, _("KiB"));
 }
 
 static gchar *gen_speed_string (gulong speed)
 {
-	gchar *retval;
+	if (speed > 1024 * 1024)
+		return g_strdup_printf ("%.1lf %s", (gdouble) speed / (1024.0 * 1024.0), _("GiB/s"));
+	if (speed > 1024)
+		return g_strdup_printf ("%.1lf %s", (gdouble) speed / 1024.0, _("MiB/s"));
+	if (speed > 0)
+		return g_strdup_printf ("%lu %s", speed, _("KiB/s"));
 
-	if (speed > 0UL) 
-	{
-		if (speed < 1024UL) {
-			retval = g_strdup_printf("%4lu Kb/s", speed);
-		} else {
-			speed /= 1024UL;
-			if (speed < 1024UL) {
-				retval = g_strdup_printf("%4lu Mb/s", speed);
-			} else {
-				speed /= 1024UL;
-				if (speed < 1024UL) {
-					retval = g_strdup_printf("%4lu Gb/s", speed);
-				} else {
-					retval = g_strdup ("");
-				}                    
-			}
-		}    
-	}
-	else
-	{
-		retval = g_strdup ("00 b/s (stalled)");
-	}
-	return retval;
+	/* Zero bytes per second in transfer speed */
+	return g_strdup (_("0 b/s (stalled)"));
 }
 
 void gui_terminate (void)
@@ -138,20 +96,19 @@ trayicon_activate (EggStatusIcon *icon, gpointer user_data);
 
 void show_status_icon (const gchar *tooltip)
 {
-	GValue is_active = {0, };
+	gboolean is_active;
 	
 	if (!cfg.use_tray_icon)
 		return;
-	g_value_init (&is_active, G_TYPE_BOOLEAN);
-	g_object_get_property (G_OBJECT(MW("main_window")),
-			       "is-active", &is_active);
+
+	g_object_get (MW("main_window"),
+		      "is-active", &is_active,
+		      NULL);
 	
-	if (g_value_get_boolean (&is_active)) {
+	if (is_active) {
 		pur_log_dbg ("Won't show notification icon: main window already focused");
-		g_value_unset(&is_active);
 		return;
-	};
-	g_value_unset(&is_active);
+	}
 	
 	if (!status_icon) {
 		gchar *trayicon_fname = g_build_filename (pixmapdir, "pureadmin-16x16.png", NULL);
@@ -178,6 +135,21 @@ void hide_status_icon (void)
 	status_icon = NULL;
 }
 
+void send_notification (const gchar *summary, const gchar *body)
+{
+	GError *err = NULL;
+	gchar *icon = g_build_filename (pixmapdir, "pureadmin-32x32.png", NULL);
+	gchar *cmd = g_strdup_printf ("notify-send \"--icon=%s\" --urgency=low \"%s\" \"%s\"", icon, summary, body);
+	if (!g_spawn_command_line_async(cmd, &err))
+	{
+		pur_log_wrn ("Couldn't send notification: %s", err->message);
+		g_error_free (err);
+		err = NULL;
+	}
+	g_free (icon);
+	g_free (cmd);
+}
+
 /* Statusbar functions, FIXME? Should contex-id be taken into account?? */
 void statusbar_push (const gchar *msg)
 {
@@ -252,7 +224,7 @@ void gui_update_server_status (void)
 	statusbar_push (msg);
 	g_free (msg);
 
-	mnu_item = MW("server_startstop");
+	mnu_item = MW("mi_server_startstop");
 	mnu_label = gtk_bin_get_child (GTK_BIN (mnu_item));
 	gtk_label_set_text_with_mnemonic (GTK_LABEL (mnu_label), lbl);
 	g_free (lbl);
@@ -263,18 +235,6 @@ void gui_update_server_status (void)
 		gtk_widget_set_sensitive (mnu_item, TRUE);
 }
 
-/* Updates the sensitivity of the menu-items depending on the state of the program */
-void gui_update_menu_sensivity (void)
-{
-	GtkWidget *mnu_item;
-
-	mnu_item = MW("close_all_connections");
-	if (!srv_has_activities ())
-		gtk_widget_set_sensitive (mnu_item, FALSE);
-	else
-		gtk_widget_set_sensitive (mnu_item, TRUE);
-}
-
 /* Creates and displays the popup-menu for the activities */
 void gui_display_activity_popup (popup_src_t source)
 {
@@ -369,9 +329,11 @@ void gui_update_info_frame (PActivity *a)
 	static GtkWidget *w_progress = NULL;
 	static GtkWidget *w_elapsed = NULL;
 	static GtkWidget *w_eta = NULL;
+	static GtkWidget *p_progress = NULL;
+	static GtkWidget *hbox_progress = NULL;
 	gchar *tmp, *num_cur_str, *num_tot_str, *speed_str, *percent;
 #define DATE_STR_LEN 50
-
+	
 	tmp = num_cur_str = num_tot_str = speed_str = percent = NULL;
 
 	if (G_UNLIKELY (!w_name))
@@ -382,18 +344,22 @@ void gui_update_info_frame (PActivity *a)
 		w_progress = MW("lbl_progress");
 		w_elapsed = MW("lbl_elapsed_time");
 		w_eta = MW("lbl_remaining_time");
+		p_progress = MW("prog_progress");
+		hbox_progress = MW("hbox_advinfo_progress");
 	}
 	if (!a)
 	{
-		/* Nothing sent, clear labels */
+		/* Nothing sent, clear labels and hide progress bar */
 		gtk_label_set_text (GTK_LABEL (w_name), "");
 		gtk_label_set_text (GTK_LABEL (w_host), "");
 		gtk_label_set_text (GTK_LABEL (w_action), "");
 		gtk_label_set_text (GTK_LABEL (w_progress), "");
 		gtk_label_set_text (GTK_LABEL (w_elapsed), "");
 		gtk_label_set_text (GTK_LABEL (w_eta), "");
+		gtk_widget_hide (p_progress);
 		return;
 	}
+
 	gtk_label_set_text (GTK_LABEL (w_name), a->username);
 	gtk_label_set_text (GTK_LABEL (w_host), a->remote_addr);
 
@@ -402,26 +368,39 @@ void gui_update_info_frame (PActivity *a)
 	g_free (tmp);
    
    
-	num_cur_str = number_separate (a->download_current_size);
+	num_cur_str = get_size_string (a->download_current_size); // number_separate (a->download_current_size);
 	speed_str   = gen_speed_string (a->speed);
 	/* If it's an upload, then we don't know the total size. */
 	if (a->state == FTPWHO_STATE_DOWNLOAD)
 	{
-		percent = g_strdup_printf ("%.2f", (double) 100 * (double) a->download_current_size / (double) a->download_total_size);
-		num_tot_str = number_separate (a->download_total_size);
-		tmp = g_strdup_printf (_("%s of %s Kb (%s%%) %s"), num_cur_str, num_tot_str, percent, speed_str);
+		gdouble fraction = (gdouble) a->download_current_size / (gdouble) a->download_total_size;
+		percent = g_strdup_printf ("%.2lf", 100.0 * fraction);
+		num_tot_str = get_size_string (a->download_total_size);
+		/* Translators: This will be come 'xx MiB of yy MiB (pp%) nn KiB/s
+		 * where 'xx' is current size, yy is total size, pp is percentage and nn is the current speed */
+		tmp = g_strdup_printf (_("%s of %s (%s%%) %s"), num_cur_str, num_tot_str, percent, speed_str);
+		gtk_progress_bar_set_text (GTK_PROGRESS_BAR(p_progress), tmp);
+		gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR(p_progress), fraction);
+		/* Show the progress bar instead of label */
+		gtk_widget_hide (w_progress);
+		gtk_widget_show (p_progress);
 	}
 	else
 	{
-		tmp = g_strdup_printf (_("%s Kb %s"), num_cur_str, speed_str);
+
+		/* Translators: This will become 'xx MiB at yy KiB/s' where xx is current
+		 * uploaded size and yy is the upload speed */
+		tmp = g_strdup_printf (_("%s at %s"), num_cur_str, speed_str);
+		gtk_label_set_text (GTK_LABEL (w_progress), tmp);
+		gtk_widget_hide (p_progress);
+		gtk_widget_show (w_progress);
 	}
-	gtk_label_set_text (GTK_LABEL (w_progress), tmp);
    
 	g_free (percent);
-	g_free (tmp);
 	g_free (num_cur_str);
 	g_free (num_tot_str);
 	g_free (speed_str);
+	g_free (tmp);
 
 	tmp = sec_to_time (a->online_since);
 	gtk_label_set_text (GTK_LABEL (w_elapsed), tmp);
@@ -436,6 +415,26 @@ void gui_update_info_frame (PActivity *a)
    
 	g_free (tmp);
 }
+GtkWidget *gui_create_system_account_menu (GList *l)
+{
+	GList *node;
+	GtkWidget *menu, *menu_item;
+   
+	menu = gtk_menu_new ();
+	node = g_list_first (l);
+	while (node)
+	{
+		SystemAccount *a = (SystemAccount *) node->data;
+		gchar *label = g_strdup_printf ("%s (%d)", a->name, a->id);
+		menu_item = gtk_menu_item_new_with_label (label);
+		g_free (label);
+		g_object_set_data (G_OBJECT(menu_item), "account_id", GINT_TO_POINTER(a->id));
+		gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
+		node = g_list_next (node);
+	}
+	gtk_widget_show_all (menu);
+	return menu;
+}
 
 GtkWidget *gui_create_menu_from_glist (GList *l)
 {
@@ -446,7 +445,11 @@ GtkWidget *gui_create_menu_from_glist (GList *l)
 	node = g_list_first (l);
 	while (node)
 	{
+//		SystemAccount *a = (SystemAccount *) node->data;
+//		gchar *label = g_strdup_printf ("%s (%d)", a->name, a->id);
 		menu_item = gtk_menu_item_new_with_label ((gchar *) node->data);
+//		g_free (label);
+//		g_object_set_data (G_OBJECT(menu_item), "account_id", GINT_TO_POINTER(a->id));
 		gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
 		node = g_list_next (node);
 	}
@@ -651,3 +654,14 @@ create_pixbuf (const gchar *filename)
 	g_free (pathname);
 	return pixbuf;
 }
+
+void show_not_yet_implemented (GtkWindow *parent)
+{
+	GtkWidget *dlg = gtk_message_dialog_new (parent,
+					      0,
+					      GTK_MESSAGE_ERROR,
+					      GTK_BUTTONS_OK,
+					      "The requested feature is not yet implemented");
+	gtk_dialog_run (GTK_DIALOG(dlg));
+	gtk_widget_destroy (dlg);
+}
