@@ -89,6 +89,7 @@ typedef enum {
 void gui_terminate (void);
 void show_status_icon (const gchar *tooltip);
 void hide_status_icon (void);
+void send_notification (const gchar *summary, const gchar *body);
 void statusbar_push (const gchar *msg);
 void statusbar_pop (void);
 
@@ -102,7 +103,9 @@ void gui_add_to_online_users (PGuiUserRow line);
 void gui_clear_activity_list (void);
 void gui_clear_users_list (void);
 void gui_update_info_frame (PActivity *a);
-GtkWidget *gui_create_menu_from_glist (GList *l);
+//GtkWidget *gui_create_menu_from_glist (GList *l);
+GtkWidget *gui_create_system_account_menu (GList *l);
+
 
 void gui_display_msgdialog_checkbox (const gchar *primary, const gchar *secondary, const gchar *icon_type, GtkWidget *parent, gboolean *show_again);
 void gui_display_msgdialog (const gchar *primary, const gchar *secondary, const gchar *icon_type, GtkWidget *parent);
@@ -115,4 +118,6 @@ void set_pixmap_directory (const gchar *directory);
 /* This is used to create the pixbufs used in the interface. */
 GdkPixbuf *create_pixbuf (const gchar *filename);
 
+void show_not_yet_implemented (GtkWindow *parent);
+
 #endif /* __GUI_HELPER_H__ */
