@@ -27,16 +27,18 @@
 #  include <config.h>
 #endif
 #define _XOPEN_SOURCE
+#include <gtk/gtk.h>
 #include <stdio.h>
 #include <unistd.h>
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
 #include <sys/types.h>
+#include <sys/wait.h>
+#include <sys/stat.h>
 #include <signal.h>
 #include <errno.h>
 #include <stdarg.h>
-#include <gtk/gtk.h>
 #include "helper.h"
 #include "usr_manager.h"
 #include "globals.h"
@@ -77,7 +79,6 @@ G_GNUC_PURE gboolean misc_str_is_only_whitespace (const gchar *s1)
 gchar **create_array (gint *argc_out, ...)
 {
 	va_list ap;
-	gint num;
 	GPtrArray *arr;
 	gchar *cur;
 
@@ -95,6 +96,29 @@ gchar **create_array (gint *argc_out, ...)
 	return (gchar **) g_ptr_array_free (arr, FALSE);
 }
 
+
+/* Spawns a command and return its output (only from stdout) as a newly allocated
+ * string. */
+gchar *misc_spawn_command (const gchar *cmdline)
+{
+	gchar *rv = NULL;
+	gboolean success = g_spawn_command_line_sync (cmdline, &rv, NULL, NULL, NULL);
+	if (!success)
+		return NULL;
+
+	return rv;
+}
+
+/* Gets the contents of /proc/<PID>/file as a newly allocated string. */
+gchar *misc_get_process_info (const gchar *pid, const gchar *file)
+{
+	gchar *path = g_build_filename ("/proc", pid, file, NULL);
+	gchar *rv = NULL;
+	g_file_get_contents (path, &rv, NULL, NULL);
+
+	return rv;
+}
+
 /* string_to_utf8: This function will do its best trying to convert the
  *		   input string to UTF-8. It will convert as many bytes as
  *		   possible, appending partial conversion with the
@@ -234,22 +258,34 @@ gchar *pur_elipzise (const gchar *orig_s, guint len)
 	return rv;
 }
 
+gchar *filename_without_extension (const gchar *filename)
+{
+	if (filename == NULL || *filename == '\0')
+		return g_strdup (filename);
+	const gchar *ext = strrchr (filename, '.');
+	if (ext == NULL)
+		return g_strdup (filename);
+	
+	return g_strndup (filename, gpointer_diff (ext, filename));
+}
+
 gboolean misc_stop_server (GError **err)
 {
 	GError *i_err = NULL;
-	gchar *cmd, *stdout, *stderr;
+	gchar *cmd, *out_stdout, *out_stderr;
 	gint ret, exitstatus;
 
 	g_return_val_if_fail (err == NULL || *err == NULL, FALSE);
 	
 	cmd = g_strdup_printf ("%s stop", cfg.cmd_startstop);
 	pur_log_nfo ("Stopping server: %s", cmd);
-	ret = g_spawn_command_line_sync (cmd, &stdout, &stderr,
+	ret = g_spawn_command_line_sync (cmd, &out_stdout, &out_stderr,
 					 &exitstatus, &i_err);
 	g_free (cmd);
-	g_free (stdout);
-	g_free (stderr);
+	g_free (out_stdout);
+	g_free (out_stderr);
 	if (!ret) {
+		exitstatus = WEXITSTATUS(exitstatus);
 		if (exitstatus != 0)
 			pur_log_wrn ("Stop server failed, child exited abnormally: %d", exitstatus);
 		g_propagate_error (err, i_err);
@@ -262,18 +298,21 @@ gboolean misc_stop_server (GError **err)
 gboolean misc_start_server (GError **err)
 {
 	GError *i_err = NULL;
-	gchar *cmd, *stdout, *stderr;
+	gchar *cmd, *out_stdout, *out_stderr;
 	gint ret, exitstatus;
 
 	g_return_val_if_fail (err == NULL || *err == NULL, FALSE);
 	
 	cmd = g_strdup_printf ("%s start", cfg.cmd_startstop);
 	pur_log_nfo ("Starting server: %s", cmd);
-	ret = g_spawn_command_line_sync (cmd, &stdout, &stderr,
+	ret = g_spawn_command_line_sync (cmd, &out_stdout, &out_stderr,
 					 &exitstatus, &i_err);
 	g_free (cmd);
-
+	/* FIXME: Do something clever with the output */
+	g_free (out_stdout);
+	g_free (out_stderr);
 	if (!ret) {
+		exitstatus = WEXITSTATUS(exitstatus);
 		if (exitstatus != 0)
 			pur_log_wrn ("Start server failed, child exited abnormally: %d", exitstatus);
 		g_propagate_error (err, i_err);
@@ -383,7 +422,7 @@ gchar *misc_get_line_beginning_with (const gchar *filename, const gchar *needle)
 gchar *misc_find_webbrowser (void)
 {
 	gchar *browser = NULL;
-	gchar *stdout = NULL;
+	gchar *output = NULL;
 
 	/* First choise: the $BROWSER environment variable */
 	if (g_getenv("BROWSER")) {
@@ -395,9 +434,9 @@ gchar *misc_find_webbrowser (void)
 	if (g_find_program_in_path("gconftool-2") &&
 	    g_spawn_command_line_sync ("gconftool-2 -g "
 				       "/desktop/gnome/url-handlers/http/enabled "
-				       "/desktop/gnome/url-handlers/http/command", &stdout, NULL, NULL, NULL))
+				       "/desktop/gnome/url-handlers/http/command", &output, NULL, NULL, NULL))
 	{
-		gchar **values = g_strsplit(g_strchomp(stdout), "\n", 2);
+		gchar **values = g_strsplit(g_strchomp(output), "\n", 2);
 		if (arr_count(values) == 2) {
 			pur_log_dbg ("Web browser found by gconftool-2: [%s], [%s])", values[0], values[1]);
 			if (strcmp (values[0], "true") == 0)
@@ -432,84 +471,11 @@ gchar *misc_find_webbrowser (void)
 	return browser;
 }
 
-static gint id_str_compare (const gchar *s1, const gchar *s2)
-{
-	/* Numeric sorting */
-	gint a, b;
-	
-	if (!s1 || !s2)
-		return -1;
-	a = atoi (s1);
-	b = atoi (s2);
-	if (a < b)
-		return -1;
-	else if (a > b)
-		return 1;
-	else
-		return 0;
-}
-
-GList *misc_get_user_ids (void)
-{
-	FILE *fl;
-	GList *retval = NULL;
-	gchar buf[LINE_MAX], **arr;
-
-	if ((fl = fopen ("/etc/passwd", "r")) == NULL)
-	{
-		pur_log_wrn ("Unable to open password file [/etc/passwd]");
-		return NULL;
-	}
-	while (fgets (buf, LINE_MAX, fl))
-	{
-		arr = g_strsplit (buf, ":", -1);
-		if (arr_count (arr) < 2)
-		{
-			g_strfreev (arr);
-			continue;
-		}
-		if (arr[0][0] && arr[2][0] &&
-		    atoi (arr[2]) > 0)
-			retval = g_list_insert_sorted (retval, g_strdup_printf ("%s (%s)", arr[2], arr[0]), (GCompareFunc) id_str_compare);
-		g_strfreev (arr);
-	}
-	fclose (fl);
-	return g_list_first (retval);
-}
-
-GList *misc_get_group_ids (void)
-{
-	FILE *fl;
-	GList *retval = NULL;
-	gchar buf[LINE_MAX], **arr;
-
-	if ((fl = fopen ("/etc/group", "r")) == NULL)
-	{
-		pur_log_wrn ("Unable to open group file [/etc/group]");
-		return NULL;
-	}
-	while (fgets (buf, LINE_MAX, fl))
-	{
-		arr = g_strsplit (buf, ":", -1);
-		if (arr_count (arr) < 2)
-		{
-			g_strfreev (arr);
-			continue;
-		}
-		if (arr[0][0] && arr[2][0] &&
-		    atoi (arr[2]) > 0)
-			retval = g_list_insert_sorted (retval, g_strdup_printf ("%s (%s)", arr[2], arr[0]), (GCompareFunc) id_str_compare);
-		g_strfreev (arr);
-	}
-	fclose (fl);
-	return g_list_first (retval);
-}
-
-gboolean misc_create_passwd_file (void)
+gboolean misc_create_passwd_file (const gchar *filename)
 {
 	FILE *f;
    
-	if ((f = fopen (cfg.pwfile, "w")) == NULL)
+	if ((f = fopen (filename, "w")) == NULL)
 	{
 		pur_log_wrn ("Could not create password file: %s", strerror (errno));
 		return FALSE;
@@ -518,69 +484,42 @@ gboolean misc_create_passwd_file (void)
 	return TRUE;
 }
 
-gboolean misc_create_system_account (gboolean create_user, gboolean create_group, GError **err)
+/* Creates a directory and all its components. Also sets the owner and group of the directory */
+/* If owner and/or group shouldn't be set, specify -1 as uid/gid. */
+gboolean misc_create_directory (const gchar *directory, gint uid, gint gid, GError **err)
 {
-	gboolean ret;
-	gchar *cmd;
-	gint child_retval;
-
+	gchar **parts;
+	gboolean rv = TRUE;
 	g_return_val_if_fail (err == NULL || *err == NULL, FALSE);
-	if (create_group && !cfg.cmd_groupadd) {
-		g_set_error (err, PUREADMIN_MAIN_ERROR,
-			     PA_MAIN_ERROR_COMMAND_NOT_FOUND,
-			     "Command 'groupadd' isn't available");
+	if (!g_path_is_absolute(directory))
+	{
+		g_set_error(err, PUREADMIN_MAIN_ERROR,
+				PA_MAIN_ERROR_FAILED,
+				"'%s' is not a valid directory", directory);
 		return FALSE;
 	}
-	if (create_user &&  !cfg.cmd_useradd) {
-		g_set_error (err, PUREADMIN_MAIN_ERROR,
-			     PA_MAIN_ERROR_COMMAND_NOT_FOUND,
-			     "Command 'useradd' isn't available");
-		return FALSE;
-	}
-	
-	if (create_group)
+	parts = g_strsplit(directory, G_DIR_SEPARATOR_S, -1);
+	GString *s = g_string_new("");
+	for (gint i = 1; parts[i] != NULL; i++)
 	{
-		cmd = g_strdup_printf ("%s ftpgroup", cfg.cmd_groupadd);
-		ret = g_spawn_command_line_sync (cmd, NULL, NULL, &child_retval, NULL);
-		g_free (cmd);
-		if (child_retval){
-			g_set_error (err, PUREADMIN_MAIN_ERROR,
-				     PA_MAIN_ERROR_FAILED,
-				     "Child 'groupadd' returned abnormally: %d", child_retval);
-			return FALSE;
-		}
-		if (!ret) {
-			g_set_error (err, PUREADMIN_MAIN_ERROR,
-				     PA_MAIN_ERROR_FAILED,
-				     "Couldn't spawn child process 'groupadd'");
-			return FALSE;
+		g_string_append_c(s, '/');
+		g_string_append(s, parts[i]);
+		if (g_file_test (s->str, G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR))
+			continue;
+		if (mkdir(s->str, 0755) != 0)
+		{
+			g_set_error(err, PUREADMIN_MAIN_ERROR,
+				PA_MAIN_ERROR_FAILED,
+				"Couldn't create directory: %s", g_strerror(errno));
+			rv = FALSE;
+			break;
 		}
 	}
-	if (create_user)
-	{
-		if (create_group || cfg.default_gid < 1)
-			cmd = g_strdup_printf ("%s -g ftpgroup -d /dev/null -s /etc ftpuser", cfg.cmd_useradd);
-		else
-			cmd = g_strdup_printf ("%s -g %d -d /dev/null -s /etc ftpuser", cfg.cmd_useradd, cfg.default_gid);
-		pur_log_dbg ("Running command [%s]", cmd);
-		ret = g_spawn_command_line_sync (cmd, NULL, NULL, &child_retval, NULL);
-		g_free (cmd);    
-
-		if (child_retval){
-			g_set_error (err, PUREADMIN_MAIN_ERROR,
-				     PA_MAIN_ERROR_FAILED,
-				     "Child 'useradd' returned abnormally: %d", child_retval);
-			return FALSE;
-		}
-		if (!ret) {
-			g_set_error (err, PUREADMIN_MAIN_ERROR,
-				     PA_MAIN_ERROR_FAILED,
-				     "Couldn't spawn child process 'useradd'");
-			return FALSE;
-		}
-	}
-   
-	return TRUE;
+	g_strfreev(parts);
+	g_string_free(s, TRUE);
+	if (rv == TRUE)
+		chown(directory, uid, gid);
+	return rv;
 }
 
 /* Crypt functions taken pure-pw distributed with pureftpd source package:
