@@ -19,7 +19,7 @@
 /*
  * Misc helperfunctions used throughout the program 
  *
- * Copyright (C) 2003 Isak Savo
+ * Copyright (C) 2003-2006 Isak Savo
  */
 #ifndef __HELPER_H__
 #define __HELPER_H__
@@ -29,10 +29,14 @@ gint arr_count (gchar **arr);
 gboolean misc_str_is_only_whitespace (const gchar *s1);
 gchar **create_array (gint *argc_out, ...);
 
+gchar *misc_spawn_command (const gchar *cmdline);
+gchar *misc_get_process_info (const gchar *pid, const gchar *file);
+
 gchar *string_to_utf8 (const gchar *str);
 gchar *string_to_vuser_locale (const char *str);
 gchar *string_from_vuser_locale (const char *str);
 
+gchar *filename_without_extension (const gchar *filename);
 gboolean misc_str_isearch (const gchar *haystack, const gchar *needle);
 
 gchar *pur_elipzise (const gchar *orig_s, guint len);
@@ -49,10 +53,9 @@ gchar *misc_get_line_beginning_with (const gchar *filename, const gchar *needle)
 
 gchar *misc_find_webbrowser (void);
 
-GList *misc_get_user_ids (void);
-GList *misc_get_group_ids (void);
-gboolean misc_create_passwd_file (void);
-gboolean misc_create_system_account (gboolean create_user, gboolean create_group, GError **err);
+gboolean misc_create_passwd_file (const gchar *filename);
+
+gboolean misc_create_directory (const gchar *directory, gint uid, gint gid, GError **err);
 gchar *misc_crypt_passwd (const gchar * const pwd);
 
 #define MISC_VALID_STRING(s) ((s != NULL && *(s) != 0))
