@@ -23,23 +23,21 @@
  */
 #ifndef __LOGFILE_H__
 #define __LOGFILE_H__
+
 typedef enum 
 {
-  LOG_INFO,
-  LOG_WARNING,
-  LOG_ERROR,
-  LOG_NOTICE,
-  LOG_DEBUG,
-  NUM_LOGTAGS
+	LOG_INFO,
+	LOG_WARNING,
+	LOG_ERROR,
+	LOG_NOTICE,
+	LOG_DEBUG,
+	NUM_LOGTAGS
 } log_textview_mode;
 
 extern GtkTextTag *logview_tags[NUM_LOGTAGS];
 
 gboolean init_logfile (void);
 void close_logfile (void);
-/* Display an errormessage explaining why logfile viewer has been disabled.
-   reason = 0 ==> FAM isn't installed,
-   reason = 1 ==> FAM isn't running. */
 void log_display_error_text (gint reason);
 void log_clear_logview (void);
 void read_logfile (void);
