@@ -24,6 +24,7 @@
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
 #endif
+#include <gtk/gtk.h>
 
 #ifdef HAVE_LIBDL
 #include <dlfcn.h>
@@ -33,7 +34,6 @@
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
-#include <gtk/gtk.h>
 #include <glade/glade.h>
 #include <errno.h>
 
@@ -51,12 +51,14 @@
 #include "eggstatusicon.h"
 
 gboolean timeout_update_activity (gpointer data);
+gboolean timeout_check_for_availability (gpointer data);
 
 static ftp_runmode_t get_ftp_runmode (void);
 static void activity_show_error_message (const gchar *errmsg);
 static void activity_show_welcome_message (void);
 static gboolean create_remaining_xmls (gpointer data);
 
+gboolean debug = FALSE;
 gboolean prog_exiting = FALSE;
 
 GladeXML *mwin_xml = NULL;
@@ -89,7 +91,7 @@ void exit_program (void)
 }
 
 /* Will be called before main() */
-static void __attribute__((constructor)) locate_dirs () 
+static void locate_dirs () 
 {
 	gbr_init(NULL);
 	datadir = gbr_find_data_dir(DATADIR);
@@ -100,13 +102,19 @@ static void __attribute__((constructor)) locate_dirs ()
 
 int main (int argc, char *argv[])
 {
-	gchar *our_path;
+	gchar *our_path = NULL;
 	const gchar *startup_id;
 	guint loghandler_id;
 	gboolean srv_comm_have_access = FALSE;
 	GError *err = NULL;
 
-	g_usleep (1000000);
+	if (g_getenv ("PUREADMIN_DEBUG") && *g_getenv ("PUREADMIN_DEBUG") != '0')
+		debug = TRUE;
+	locate_dirs();
+	pur_log_dbg ("Binary: %s", selfname);
+	pur_log_dbg ("Datadir: %s", datadir);
+	pur_log_dbg ("Pixmaps: %s", pixmapdir);
+	pur_log_dbg ("Documents: %s", docdir);
 #ifdef ENABLE_NLS
 	our_path = gbr_find_locale_dir(LOCALEDIR);
 	bindtextdomain (GETTEXT_PACKAGE, our_path);
@@ -117,7 +125,8 @@ int main (int argc, char *argv[])
 	startup_id = g_getenv ("DESKTOP_STARTUP_ID");
 	gtk_init (&argc, &argv);
 	/* Restore DESKTOP_STARTUP_ID since, appearently, gtk_init() sometimes clears it */
-	g_setenv ("DESKTOP_STARTUP_ID", startup_id, TRUE);
+	// FIXME: Do we need to check if setenv() is available?
+	setenv ("DESKTOP_STARTUP_ID", startup_id, TRUE);
 	
 	if (getuid() != 0)
 	{
@@ -129,6 +138,7 @@ int main (int argc, char *argv[])
 			return 0;
 		}
 	}
+	
 #ifndef G_THREADS_ENABLED
 	g_error ("Threading is not enabled, this will not work!");
 #endif
@@ -136,6 +146,12 @@ int main (int argc, char *argv[])
 
 	our_path = g_build_filename (datadir, PACKAGE, "mainwindow.glade", NULL);
 	mwin_xml = glade_xml_new (our_path, NULL, NULL);
+	if (!mwin_xml)
+	{
+		g_warning ("Couldn't load glade file '%s', please make sure it is installed correctly.", our_path);
+		g_print ("PureAdmin will now terminate since the graphical interface couldn't be created\n");
+		return -1;
+	}
 	g_free (our_path);
 	
 	
@@ -172,9 +188,8 @@ int main (int argc, char *argv[])
   
 	gtk_widget_show (MW("main_window"));
 
-	/* Update statusbar with status of PureFTPd and make the menu-widgets sensitive where appropiate */
+	/* Update statusbar with status of PureFTPd */
 	gui_update_server_status ();
-	gui_update_menu_sensivity ();
 	 
 	err = NULL;
 	srv_comm_have_access = srv_try_get_activities (&err);
@@ -187,13 +202,17 @@ int main (int argc, char *argv[])
 	} else {
 		activity_show_error_message (err->message);
 		pur_log_wrn ("Don't have access to server activities: %s", err->message);
+		pur_log_wrn ("Will re-check every 3 seconds if activities are available again");
+		g_timeout_add (3000, (GSourceFunc) timeout_check_for_availability, NULL);
+
 		g_error_free (err);
 		err = NULL;
 	}
 	pur_log_nfo ("PureAdmin started!");
-
+	
 	g_thread_create ((GThreadFunc) srv_activity_thread,
 			 NULL, FALSE, &err);
+	/* Add this anyway, since the functionality to get activities could recover */
 
 	if (err){
 		pur_log_err ("Couldn't create activity thread: %s", err->message);
@@ -203,10 +222,14 @@ int main (int argc, char *argv[])
 	else
 		pur_log_dbg ("Activity thread created successfully");
 	glade_xml_signal_autoconnect (mwin_xml);
-
+	
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (MW("chk_show_hide_info")), cfg.show_advinfo);
+	/* Need to call this manually, since the event handler will only be called if the button state changes */
+	gui_update_info_frame (NULL);
+	
 	g_idle_add ((GSourceFunc) create_remaining_xmls, NULL);
 	//create_remaining_xmls(NULL);
-	gtk_widget_show_all (MW("main_window"));
+	gtk_widget_show (MW("main_window"));
 	gtk_main ();
 	/* make the last messages go to stderr */
 	g_log_remove_handler (NULL, loghandler_id);
@@ -218,19 +241,27 @@ int main (int argc, char *argv[])
 static ftp_runmode_t get_ftp_runmode (void)
 {
 	gchar *s = misc_get_line_beginning_with ("/etc/inetd.conf", "ftp");
-	
+	gchar *output = NULL;
 	if (s && strstr (s, "pure-ftp")) {
 		g_free (s);
 		return RUNMODE_INETD;
 	}
 	g_free (s);
 
-	s = misc_find_file_in_dirs ("pure-ftpd.pid", "/var/run /var/run/pure-ftpd");
-	if (s) {
-		/* FIXME: Maybe we should check for stale PID-file, but
-		 * i don't think it matters that much. */
-		g_free (s);
-		return RUNMODE_STANDALONE;
+	if ((output = misc_spawn_command ("pgrep pure-ftpd")) != NULL)
+	{
+		gchar **pids = g_strsplit (output, "\n", -1);
+		gint i = 0;
+		while (pids && pids[i])
+		{
+			gchar *pid_cmdline = misc_get_process_info (pids[i], "cmdline");
+			if (strncmp (pid_cmdline, "pure-ftpd (SERVER)", strlen("pure-ftpd (SERVER)")) == 0)
+			{
+				g_free (pid_cmdline);
+				return RUNMODE_STANDALONE;
+			}
+			i++;
+		}
 	}
 
 	return RUNMODE_STOPPED;
@@ -248,6 +279,7 @@ void activity_show_welcome_message (void)
 	model = gtk_tree_view_get_model (GTK_TREE_VIEW (tree_activities));
 	//sel = gtk_tree_view_get_selection (GTK_TREE_VIEW (tree_activities));
 	//gtk_tree_selection_set_mode (sel, GTK_SELECTION_NONE);
+	gui_clear_activity_list ();
 
 	icon = gtk_widget_render_icon (tree_activities, "gtk-dialog-info", GTK_ICON_SIZE_MENU, "");
 
@@ -302,3 +334,17 @@ static gboolean create_remaining_xmls (gpointer data)
 	/* We're done, don't call us again! */
 	return FALSE;
 }
+
+gboolean timeout_check_for_availability (gpointer data)
+{
+	if (srv_try_get_activities (NULL))
+	{
+		activity_show_welcome_message ();
+		g_timeout_add (1000, (GSourceFunc) timeout_update_activity, NULL);
+		pur_log_nfo ("Access to server activites finally granted!");
+		/* Our work is done, remove this function from the calling list */
+		return FALSE;
+	}
+	/* Re-try again in a couple of seconds */
+	return TRUE;
+}
