@@ -35,6 +35,8 @@
 
 #include "globals.h"
 #include "gui_helper.h"
+#include "mainwin.h"
+#include "prereq_usrmanager.h"
 #include "usr_manager.h"
 #include "helper.h"
 #include "prefwin.h"
@@ -55,7 +57,6 @@ static void update_adv_info (void)
 	GtkTreeIter iter;
 	gboolean visible = TRUE, selected;
 	gint id;
-	GValue val;
 	PActivity *act;
 
 	if (G_UNLIKELY (!tree_activities))
@@ -66,10 +67,9 @@ static void update_adv_info (void)
 		sel = gtk_tree_view_get_selection (GTK_TREE_VIEW (tree_activities));
 	}
 
-	memset (&val, 0, sizeof (GValue));
-	g_value_init (&val, G_TYPE_BOOLEAN);
-	g_object_get_property (G_OBJECT (adv_info), "visible", &val);
-	visible = g_value_get_boolean (&val);
+	g_object_get (adv_info,
+		      "visible", &visible,
+		      NULL);
 	selected = gtk_tree_selection_get_selected (sel, &model, &iter);
 	if (visible)
 	{
@@ -86,10 +86,13 @@ static void update_adv_info (void)
 		else
 			gui_update_info_frame (NULL);  /* Clear advanced info labels */
 	}
+	
 }
 
 
-static gint gui_tree_act_get_selected_id (void)
+/* Gets the ID of the selected activity and stores it in the location pointed to by output_id.
+ * Returns true if the id was found, otherwise false*/
+static gboolean gui_tree_act_get_selected_id (guint *output_id)
 {
 	static GtkWidget *tree_activities = NULL;
 	static GtkTreeModel *model = NULL;
@@ -106,11 +109,12 @@ static gint gui_tree_act_get_selected_id (void)
 	}
 	selected = gtk_tree_selection_get_selected (sel, &model, &iter);
 	if (!selected) /* Nothing is selected! */
-		return -1;
+		return FALSE;
 	gtk_tree_model_get (model, &iter, 
 			    COL_ACT_ID, &id,
 			    -1);
-	return id;
+	*output_id = id;
+	return TRUE;
 }
 
 static gboolean gui_tree_usr_get_selected_user (gchar **user, gchar **host)
@@ -172,16 +176,19 @@ gboolean on_mainwin_focus_in_event (GtkWidget *widget,
 				    gpointer user_data)
 {
 	hide_status_icon();
+	return FALSE;
 }
 
 void
-show_hide_advanced_info                (GtkToggleButton *togglebutton,
-                                        gpointer         user_data)
+show_hide_advanced_info_cb (GtkToggleButton *togglebutton, gpointer user_data)
 {
 	GtkWidget *adv_info = MW("tbl_advanced_info");
-   
-	if (gtk_toggle_button_get_active (togglebutton))
+	cfg.show_advinfo = gtk_toggle_button_get_active (togglebutton);
+	if (cfg.show_advinfo)
+	{
 		gtk_widget_show (adv_info);
+		update_adv_info();
+	}
 	else
 		gtk_widget_hide (adv_info);
 }
@@ -190,7 +197,17 @@ void
 on_tree_activity_cursor_changed        (GtkTreeView     *treeview,
                                         gpointer         user_data)
 {
+	guint id;
 	update_adv_info ();
+	
+	
+	if (gui_tree_act_get_selected_id(&id))
+	{
+		PActivity *act = srv_get_activity_with_id (id);
+		g_return_if_fail (act != NULL);
+		user_list_select_user(act->username, act->remote_addr);
+		pactivity_free (act);
+	}
 }
 
 
@@ -201,9 +218,12 @@ on_btn_user_manager_clicked            (GtkButton       *button,
 	GtkWidget *dlg_usrman;
 	GtkWidget *mainwin = MW("main_window");
 
-	if (!usr_manager_check_prerequisites ())
-		return;
-
+	if (prereq_are_all_requirements_met() == FALSE)
+	{
+		gboolean success = prereq_show_dialog(GTK_WINDOW(mainwin));
+		if (!success)
+			return;
+	}
 	dlg_usrman = usr_init_usermanager_window (mainwin);
 	
 	gtk_window_present (GTK_WINDOW (dlg_usrman));
@@ -211,7 +231,7 @@ on_btn_user_manager_clicked            (GtkButton       *button,
 
 
 void
-menu_quit                              (GtkMenuItem     *menuitem,
+menu_quit_cb                              (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
 	exit_program ();
@@ -220,8 +240,7 @@ menu_quit                              (GtkMenuItem     *menuitem,
 
 
 void
-menu_preferences                       (GtkMenuItem     *menuitem,
-                                        gpointer         user_data)
+menu_preferences_cb (GtkMenuItem *menuitem, gpointer user_data)
 {
 	GtkWidget *prefwin = pref_init_prefwindow (MW("main_window"));
 	
@@ -230,8 +249,7 @@ menu_preferences                       (GtkMenuItem     *menuitem,
 
 
 void
-menu_srv_startstop                     (GtkMenuItem     *menuitem,
-                                        gpointer         user_data)
+menu_srv_startstop_cb (GtkMenuItem *menuitem, gpointer user_data)
 {
 	GError *err = NULL;
 	int ret;
@@ -289,8 +307,65 @@ menu_srv_startstop                     (GtkMenuItem     *menuitem,
 
 
 void
-menu_close_connections                 (GtkMenuItem     *menuitem,
-                                        gpointer         user_data)
+menu_user_manager_cb (GtkMenuItem *menuitem, gpointer user_data)
+{
+	GtkWidget *dlg_usrman;
+	
+	if (!prereq_are_all_requirements_met())
+		if (!prereq_show_dialog(GTK_WINDOW(MW("main_window"))))
+			return;	
+	dlg_usrman = usr_init_usermanager_window (MW("main_window"));
+	gtk_window_present (GTK_WINDOW (dlg_usrman));
+}
+
+void
+menu_connections_cb (GtkMenuItem *menuitem, gpointer user_data)
+{
+	GtkTreeView *act_tree = GTK_TREE_VIEW (MW("tree_activity"));
+	GtkTreeSelection *sel = gtk_tree_view_get_selection (act_tree);
+	GtkTreeIter iter;
+	GtkTreeModel *model = NULL;
+	
+       	gtk_widget_set_sensitive (MW("mi_close_all"), srv_has_activities ());
+
+	gboolean selected = gtk_tree_selection_get_selected (sel, NULL, NULL);
+	gtk_widget_set_sensitive (MW("mi_close_tx"), selected);
+	gtk_widget_set_sensitive (MW("mi_notify"), selected);
+
+
+	gchar *text;
+	sel = gtk_tree_view_get_selection (GTK_TREE_VIEW (MW("tree_users")));
+	selected = gtk_tree_selection_get_selected (sel, &model, &iter);
+	gtk_widget_set_sensitive (MW("mi_disconnect"), selected);
+	if (selected)
+	{
+		gchar *username = NULL;
+		gtk_tree_model_get (model, &iter,
+				    COL_USR_USER, &username,
+				    -1);
+		text  = g_strconcat (_("Disconnect user"), " ", username, NULL);
+	}
+	else
+		text = g_strdup (_("Disconnect user"));
+	GtkWidget *label = gtk_bin_get_child (GTK_BIN(MW("mi_disconnect")));
+	gtk_label_set_text (GTK_LABEL(label), text);
+	g_free (text);
+}
+
+void
+menu_close_tx_cb (GtkMenuItem *menuitem, gpointer user_data)
+{
+	show_not_yet_implemented (GTK_WINDOW(MW("main_window")));
+}
+
+void
+menu_disconnect_user_cb (GtkMenuItem *menuitem, gpointer user_data)
+{
+	show_not_yet_implemented (GTK_WINDOW(MW("main_window")));
+}
+
+void
+menu_close_connections_cb (GtkMenuItem *menuitem, gpointer user_data)
 {
 	GList *act_list, *head;
 	PActivity *activity;
@@ -311,24 +386,14 @@ menu_close_connections                 (GtkMenuItem     *menuitem,
 	free_activity_list (head);
 }
 
-
-
 void
-menu_user_manager                      (GtkMenuItem     *menuitem,
-                                        gpointer         user_data)
+menu_notify_cb  (GtkMenuItem *menuitem, gpointer user_data)
 {
-	GtkWidget *dlg_usrman;
-	
-	if (!usr_manager_check_prerequisites ())
-		return;
-	dlg_usrman = usr_init_usermanager_window (MW("main_window"));
-	gtk_window_present (GTK_WINDOW (dlg_usrman));
+	show_not_yet_implemented (GTK_WINDOW(MW("main_window")));
 }
 
-
 void
-menu_about                             (GtkMenuItem     *menuitem,
-                                        gpointer         user_data)
+menu_about_cb (GtkMenuItem *menuitem, gpointer user_data)
 {
 	GtkWidget *about, *title;
 	about = DW("dlg_about");
@@ -347,12 +412,12 @@ menu_about                             (GtkMenuItem     *menuitem,
 
 
 void
-menu_help                              (GtkMenuItem     *menuitem,
-                                        gpointer         user_data)
+menu_help_cb (GtkMenuItem *menuitem, gpointer user_data)
 {
 	hlp_show_help (HLP_ITEM_INDEX);
 }
 
+
 static void open_in_browser (const gchar *url)
 {
 	gchar *browser = misc_find_webbrowser();
@@ -387,14 +452,14 @@ static void open_in_browser (const gchar *url)
 	}
 }
 
-void menu_online_support (GtkMenuItem *menuitem, gpointer user_data)
+void menu_online_support_cb (GtkMenuItem *menuitem, gpointer user_data)
 {
 	const char *url = "http://purify.sourceforge.net/forums/";
 
 	open_in_browser (url);
 }
 
-void menu_online_documentation (GtkMenuItem *menuitem, gpointer user_data)
+void menu_online_documentation_cb (GtkMenuItem *menuitem, gpointer user_data)
 {
 	const char *url = "http://purify.sourceforge.net/index.php?page=documentation";
 
@@ -462,18 +527,16 @@ on_tree_users_button_press_event       (GtkWidget       *widget,
 	return FALSE;
 }
 
-
 void
 on_popup_close_tx                      (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
-	gint id;
+	guint id;
 	PActivity *act;
 
 	if (popup_source == POPUP_FROM_ACTTREE)
 	{
-		id = gui_tree_act_get_selected_id ();
-		if (id < 0)
+		if (!gui_tree_act_get_selected_id (&id))
 		{
 			pur_log_wrn ("Unable to fetch information about selected activity");
 			return;
@@ -496,14 +559,14 @@ on_popup_close_all_conns               (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
 	gchar *user, *host;
-	gint id, success;
+	guint id;
+	gboolean success;
 	PActivity *act;
 	GList *conns, *head;
 
 	if (popup_source == POPUP_FROM_ACTTREE)
 	{
-		id = gui_tree_act_get_selected_id ();
-		if (id < 0)
+		if (!gui_tree_act_get_selected_id (&id))
 		{
 			pur_log_wrn ("Unable to fetch information about selected activity\n");
 			return;
@@ -546,15 +609,14 @@ on_popup_close_all_conns               (GtkMenuItem     *menuitem,
 
 void on_popup_show_userinfo (GtkMenuItem *menuitem, gpointer user_data)
 {
-	gint id;
+	guint id;
 	gchar *user, *host;
 	PActivity *act;
 	GtkWidget *dlg_usrman;
 	
 	if (popup_source == POPUP_FROM_ACTTREE)
 	{
-		id = gui_tree_act_get_selected_id ();
-		if (id < 0)
+		if (!gui_tree_act_get_selected_id (&id))
 		{
 			pur_log_wrn ("Unable to fetch information about selected activity\n");
 			return;
@@ -573,7 +635,9 @@ void on_popup_show_userinfo (GtkMenuItem *menuitem, gpointer user_data)
 	}
 	pur_log_dbg ("Showing userinfo for %s\n", user);
    
-	usr_manager_check_prerequisites ();
+	if (!prereq_are_all_requirements_met())
+		if(!prereq_show_dialog(GTK_WINDOW(MW("main_window"))))
+			return;
 
 	dlg_usrman = usr_init_usermanager_window (MW("main_window"));
 	gtk_window_set_transient_for (GTK_WINDOW(dlg_usrman), GTK_WINDOW (MW("main_window")));
@@ -586,8 +650,7 @@ void on_popup_show_userinfo (GtkMenuItem *menuitem, gpointer user_data)
 
 
 void
-on_view_debug_console_activate         (GtkMenuItem     *menuitem,
-                                        gpointer         user_data)
+menu_debug_console_cb (GtkMenuItem *menuitem, gpointer user_data)
 {
 	gtk_window_present (GTK_WINDOW (MW("dlg_debug")));
 }
@@ -638,6 +701,3 @@ on_btn_logview_reread_clicked          (GtkButton       *button,
 	close_logfile ();
 	init_logfile ();
 }
-
-
-
