@@ -37,6 +37,7 @@
 #include "gui_helper.h"
 #include "logfile.h"
 #include "globals.h"
+#include "system_accounts.h"
 #include "dirbrowser.h"
 
 static void prefwin_menu_gid_changed (GtkOptionMenu *optionmenu, gpointer user_data);
@@ -77,6 +78,7 @@ prefwin_widgets *prefwin_get_widgets (void)
 		w->chk_save_window_geo = PW("chk_save_window_geo");
 		w->chk_use_system_encoding = PW("chk_use_system_encoding");
 		w->chk_use_tray_icon = PW("chk_use_tray_icon");
+		w->chk_always_notify = PW("chk_always_notify");
 		w->btn_browse_purepw = PW("btn_browse_purepw");
 		w->btn_browse_pw = PW("btn_browse_pw");
 		w->btn_browse_pdb = PW("btn_browse_pdb");
@@ -118,30 +120,31 @@ GtkWidget *pref_init_prefwindow (GtkWidget *parent)
 	w = prefwin_get_widgets ();
    
 	/* GID/UID Option menus */
-	id = misc_get_user_ids ();
-	menu = gui_create_menu_from_glist (id);
+	id = sys_get_user_ids ();
+	menu = gui_create_system_account_menu(id);
 	gtk_option_menu_remove_menu (GTK_OPTION_MENU (w->menu_uid));
 	gtk_option_menu_set_menu (GTK_OPTION_MENU (w->menu_uid), menu);
+	/* Keep the list attached to the menu so that it's available in the callbacks.. */
 	g_object_set_data (G_OBJECT (w->menu_uid), "items", id);
 	i = 0;
 	while (id)
 	{
-		if (cfg.default_uid == (guint) atoi ((gchar *) id->data))
+		if (cfg.default_uid == ((SystemAccount*) id->data)->id)
 			break;
 		i++;
 		id = g_list_next (id);
 	}
 	gtk_option_menu_set_history (GTK_OPTION_MENU (w->menu_uid), i);
 
-	id = misc_get_group_ids ();
-	menu = gui_create_menu_from_glist (id);
+	id = sys_get_group_ids ();
+	menu = gui_create_system_account_menu(id);
 	gtk_option_menu_remove_menu (GTK_OPTION_MENU (w->menu_gid));
 	gtk_option_menu_set_menu (GTK_OPTION_MENU (w->menu_gid), menu);
-	g_object_set_data (G_OBJECT (w->menu_gid), "items", id);
+	g_object_set_data_full (G_OBJECT (w->menu_gid), "items", id, (GDestroyNotify) system_account_list_free);
 	i = 0;
 	while (id)
 	{
-		if (cfg.default_gid == (guint) atoi ((gchar *) id->data))
+		if (cfg.default_gid == ((SystemAccount*) id->data)->id)
 			break;
 		i++;
 		id = g_list_next (id);
@@ -173,6 +176,7 @@ GtkWidget *pref_init_prefwindow (GtkWidget *parent)
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w->chk_resolve), cfg.resolve_hostnames);
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w->chk_save_window_geo), cfg.save_window_geometry);
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w->chk_use_tray_icon), cfg.use_tray_icon);
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w->chk_always_notify), cfg.always_notify_activities);
 	g_signal_connect ((gpointer) w->chk_use_system_encoding, "toggled",
 			  G_CALLBACK (prefwin_chkencoding_toggled),
 			  NULL);
@@ -214,7 +218,10 @@ GtkWidget *pref_init_prefwindow (GtkWidget *parent)
 	g_signal_connect ((gpointer) w->chk_use_tray_icon, "toggled",
 			  G_CALLBACK (prefwin_chkbtn_toggled),
 			  "use_tray_icon");
-
+	g_signal_connect ((gpointer) w->chk_always_notify, "toggled",
+			  G_CALLBACK (prefwin_chkbtn_toggled),
+			  "always_notify_activities");
+	
 	g_signal_connect ((gpointer) w->chk_save_window_geo, "toggled",
 			  G_CALLBACK (prefwin_chkbtn_toggled),
 			  "save_window");
@@ -316,6 +323,8 @@ static void prefwin_chkbtn_toggled (GtkToggleButton *togglebutton, gpointer user
 			hide_status_icon ();
 		cfg.use_tray_icon = toggled;
 	}
+	else if (g_str_equal (arg, "always_notify_activities"))
+		cfg.always_notify_activities = toggled;
 }
 
 static void prefwin_chkencoding_toggled (GtkToggleButton *togglebutton, gpointer user_data)
