@@ -33,6 +33,8 @@
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <sys/stat.h>
+#include <sys/mman.h>
+#include <sys/wait.h>
 #include <fcntl.h>
 #include <unistd.h>
 #include <netdb.h>
@@ -51,32 +53,43 @@
 GStaticMutex alist_mutex = G_STATIC_MUTEX_INIT;
 GList *cur_activities = NULL; 
 
-/* Generates a random unique ID and returns it.
+/* Generates a unique id from the provided hash string. In most cases,
+ * this will be the same as returned from g_str_hash, but this function
+ * also guarantees that the id is unique in the provided list of activities.
+ *
  * Theoreticly this could lead to a eternal loop, but it shouldn't
  * in practice! (Unless there are a couple of billion entries in the list)
  */
 static
-gint32 _list_generate_unique_id (void)
+guint _list_generate_unique_id (const gchar *hashstr, GList *list)
 {
 	GList *node;
-	GRand *number;
-	gint32 r_num, found;
-   
-	number = g_rand_new ();
+	gboolean already_exists;
+	guint current;
+
+	/* This should in most case generate a number that is unique */
+	current = g_str_hash (hashstr);
 	do
 	{
-		found = TRUE;
-		r_num = g_rand_int_range (number, 0, G_MAXINT32);
-		node = g_list_first (cur_activities);
-		while (node || !found)
+		already_exists = FALSE;
+		node = g_list_first (list);
+		while (node && already_exists == FALSE)
 		{
-			if (((PActivity *) node->data)->id == r_num)
-				found = FALSE;
+			guint id = ((PActivity *) node->data)->id;
+
+			if (id > current)
+				/* The list is sorted ascending by ID, so this shortcut is valid */
+				return current;
+
+			if (id == current)
+				already_exists = TRUE;
 			node = g_list_next (node);
 		}
-	} while (!found);
-	g_rand_free (number);
-	return r_num;
+		if (G_UNLIKELY (already_exists))
+			current += g_random_int();
+	} while (already_exists);
+	
+	return current;
 }
 
 /* Parses a line separated by pipes ('|'), and returns an array of strings containing all the fields */
@@ -86,7 +99,7 @@ gchar **ftpwho_parse_line (gchar *line)
 	gchar **retval;
 	gchar *field;
 	gboolean done = FALSE;
-	gint arr_count, i, j, line_length;
+	gint array_count, i, j, line_length;
 	
 	if (!line || !strstr (line, "|"))
 		return g_strsplit ("", "|", -1);
@@ -98,7 +111,7 @@ gchar **ftpwho_parse_line (gchar *line)
 	i = 0;
 	line_length = strlen (line);
 	field = g_new0 (gchar, line_length);
-	for (arr_count = 0; arr_count < 12 && i < line_length; arr_count++)
+	for (array_count = 0; array_count < 12 && i < line_length; array_count++)
 	{
 		done = FALSE;
 		j = 0;
@@ -114,7 +127,7 @@ gchar **ftpwho_parse_line (gchar *line)
 				field[j++] = line[i];
 			i++;
 		}
-		retval[arr_count] = g_strdup (field);
+		retval[array_count] = g_strdup (field);
 		memset (field, '\0', j);
 	}
 	g_free (field);
@@ -147,6 +160,20 @@ gboolean has_initialized (gboolean set)
 	return rv;
 }
 
+static
+gint cmp_activities_by_id (const PActivity *a1, const PActivity *a2)
+{
+	if (G_UNLIKELY (a1 == NULL || a2 == NULL))
+		return 0;
+
+	if (a1->id < a2->id)
+		return 1;
+	if (a1->id == a2->id)
+		return 0;
+	return -1;
+}
+
+	
 static
 gint lst_find_user (gconstpointer listdata, gconstpointer searchdata)
 {
@@ -250,15 +277,15 @@ void update_activities (void)
 		activity->download_total_size = atoi (entries[9]);
 		activity->percentage = atoi (entries[10]);
 		activity->speed = atoi (entries[11]);
-		activity->id = _list_generate_unique_id ();
-
-		tmp_alist = g_list_append (tmp_alist, activity);
+		gchar *hash_string = g_strconcat (activity->username, activity->remote_addr, activity->filename, entries[3], NULL);
+		activity->id = _list_generate_unique_id (hash_string, tmp_alist);
+		tmp_alist = g_list_insert_sorted (tmp_alist, activity, (GCompareFunc) cmp_activities_by_id);
+		//tmp_alist = g_list_append (tmp_alist, activity);
 
 		g_strfreev (entries);
 		lineno++;
 	}
 	g_strfreev (lines);
-
 	/* Clear the real (old) list in replace it with the newly fetched list */
 	g_static_mutex_lock (&alist_mutex);
 	g_list_foreach (cur_activities, (GFunc) pactivity_free, NULL);
@@ -374,7 +401,7 @@ gboolean srv_has_activities (void)
 
 gboolean srv_try_get_activities (GError **err)
 {
-	gchar *output, *cmd;
+	gchar *output, *err_output, *cmd;
 	gint ret;
 	gint exit_status;
 
@@ -391,7 +418,7 @@ gboolean srv_try_get_activities (GError **err)
 	cmd = g_strdup_printf ("%s -s -H", cfg.cmd_ftpwho);
 	ret = g_spawn_command_line_sync (cmd,
 					 &output,
-					 NULL,
+					 &err_output,
 					 &exit_status,
 					 NULL);
 	pur_log_dbg ("Running command: [%s]", cmd);
@@ -406,20 +433,30 @@ gboolean srv_try_get_activities (GError **err)
 		return FALSE;
 	}
 	g_free (cmd);
+	exit_status = (signed char) WEXITSTATUS(exit_status);
 	if (exit_status)
 	{
-		if (strstr ("root", output))
+		if (strstr (output, "root"))
 			g_set_error (err,
 				     PUREADMIN_SRV_ERROR,
 				     PA_SRV_ERROR_PERMISSION_DENIED,
 				     _("Permission denied"));
 		else
-			g_set_error (err,
+		{
+			if (exit_status == -1)
+				g_set_error (err,
 				     PUREADMIN_SRV_ERROR,
 				     PA_SRV_ERROR_FAILED,
-				     _("Pure-ftpwho exited abnormally: %d. Output was: %s"),
-				     exit_status, output);
+				     _("Resource temporarily unavailable."));
+			else
+				g_set_error (err,
+				     PUREADMIN_SRV_ERROR,
+				     PA_SRV_ERROR_FAILED,
+				     _("Pure-ftpwho exited abnormally: %d. Output was: '%s' and '%s' "),
+				     exit_status, output, err_output);
+		}
 		g_free (output);
+		g_free (err_output);
 		return FALSE;
 	}
 	g_free (output);
@@ -436,13 +473,12 @@ G_GNUC_NORETURN gpointer srv_activity_thread (gpointer data)
 }
 
 /* Returns the PActivity that has the provided id number.
- * The returned value should be free'd when no longer used */
-PActivity *srv_get_activity_with_id (gint32 id)
+ * The returned value should be free'd using pactivity_free when no longer used */
+PActivity *srv_get_activity_with_id (guint id)
 {
 	GList *node = NULL;
 	PActivity *act = NULL, *rv;
 	gboolean found = FALSE;
-
 	g_static_mutex_lock (&alist_mutex);
 	node = g_list_first (cur_activities);
 	while (node && !found)
@@ -454,7 +490,6 @@ PActivity *srv_get_activity_with_id (gint32 id)
 	}
 	rv = pactivity_dup (act);
 	g_static_mutex_unlock (&alist_mutex);
-	
 	if (found)
 		return rv;
 	else
@@ -492,6 +527,7 @@ GList *srv_get_connected_users (GList *activities)
 	GList *retval = NULL, *node;
 	POnlineUser *user = NULL;
 
+	g_static_mutex_lock (&alist_mutex);
 	activities = g_list_first (activities);
 	while (activities)
 	{
@@ -509,5 +545,6 @@ GList *srv_get_connected_users (GList *activities)
 		}
 		activities = g_list_next (activities);
 	}
+	g_static_mutex_unlock (&alist_mutex);
 	return g_list_first (retval);
 }
