@@ -40,18 +40,18 @@ typedef enum {
 
 /* This is the structs returned to the GUI */
 typedef struct Activity_ {
-  FTPWhoEntryState state;
-  gchar *remote_addr;
-  //  gchar date;
-  gint percentage;
-  gint online_since;
-  gint download_total_size;
-  gint download_current_size;
-  gchar *username;
-  gchar *filename;
-  guint pid;		/* The process id of the server handling the specific activity. So that we can SIGTERM it!! */
-  gint speed;
-  gint32 id;	    /* An Unique ID for the activity */
+	FTPWhoEntryState state;
+	gchar *remote_addr;
+	//  gchar date;
+	gint percentage;
+	gint online_since;
+	gint download_total_size;
+	gint download_current_size;
+	gchar *username;
+	gchar *filename;
+	guint pid;		/* The process id of the server handling the specific activity. So that we can SIGTERM it!! */
+	gint speed;
+	guint id;	    /* An Unique ID for the activity */
 } PActivity;
 
 typedef struct OnlineUser_ {
@@ -84,7 +84,7 @@ gboolean srv_try_get_activities (GError **err);
 GList *srv_get_activities (void);
 GList *srv_get_connected_users (GList *activities);
 GList *srv_get_pids_for_user_host (const gchar *user, const gchar *host);
-PActivity *srv_get_activity_with_id (gint32 id);
+PActivity *srv_get_activity_with_id (guint id);
 void srv_terminate (void);
 
 #endif /* __SRV_COMM_H__ */
