@@ -35,6 +35,7 @@
 #include "gui_helper.h"
 #include "globals.h"
 #include "helper.h"
+#include "system_accounts.h"
 #include "usr_manager.h"
 #include "purepw.h"
 
@@ -207,17 +208,18 @@ static PWInfo *usr_widgets_get_info (void)
 	}
 
 	/* UID / GID -- Needs conversion from menuitems to int */
+	SystemAccount *tmp_account;
 	sel = gtk_option_menu_get_history (GTK_OPTION_MENU (w->e_uid));
 	items = g_object_get_data (G_OBJECT (w->e_uid), "items");
-	tmp = (gchar *) (g_list_nth (items, sel)->data);
-	if (tmp)
-		i->uid = atoi (tmp);
+	tmp_account = (SystemAccount*) g_list_nth (items, sel)->data;
+	if (tmp_account)
+		i->uid = tmp_account->id;
    
 	sel = gtk_option_menu_get_history (GTK_OPTION_MENU (w->e_gid));
 	items = g_object_get_data (G_OBJECT (w->e_gid), "items");
-	tmp = (gchar *) (g_list_nth (items, sel)->data);
-	if (tmp)
-		i->gid = atoi (tmp);
+	tmp_account = (SystemAccount*) g_list_nth (items, sel)->data;
+	if (tmp_account)
+		i->gid = tmp_account->id;
    
 	/* Time -- Needs conversion from char to ints */
 	tmp = (gchar *) gtk_entry_get_text (GTK_ENTRY (w->timelimit));
@@ -246,18 +248,19 @@ static void setup_optmenus (void)
 	GList *IDs;
    
 	opt_menu = UW("optmenu_uid");
-	IDs = misc_get_user_ids ();
-	menu = gui_create_menu_from_glist (IDs);
+	IDs = sys_get_user_ids();
+	menu = gui_create_system_account_menu(IDs);
 	gtk_option_menu_remove_menu (GTK_OPTION_MENU (opt_menu));
 	gtk_option_menu_set_menu (GTK_OPTION_MENU (opt_menu), menu);
-	g_object_set_data (G_OBJECT (opt_menu), "items", IDs);
+	/* Keep the list attached to the menu for fast access in the callbacks */
+	g_object_set_data_full (G_OBJECT (opt_menu), "items", IDs, (GDestroyNotify) system_account_list_free);
 
 	opt_menu = UW("optmenu_gid");
-	IDs = misc_get_group_ids ();
-	menu = gui_create_menu_from_glist (IDs);
+	IDs = sys_get_group_ids();
+	menu = gui_create_system_account_menu(IDs);
 	gtk_option_menu_remove_menu (GTK_OPTION_MENU (opt_menu));
 	gtk_option_menu_set_menu (GTK_OPTION_MENU (opt_menu), menu);
-	g_object_set_data (G_OBJECT (opt_menu), "items", IDs);
+	g_object_set_data_full (G_OBJECT (opt_menu), "items", IDs, (GDestroyNotify) system_account_list_free);
    
 	g_object_set_data (G_OBJECT (UW("usermanager")), "editing", GINT_TO_POINTER (FALSE));
 }
@@ -548,7 +551,7 @@ gboolean usr_widgets_set_info (gchar *user)
 	found = FALSE;
 	while (items)
 	{
-		if (info->uid == (guint) atoi ((gchar *) items->data))
+		if (info->uid == ((SystemAccount *) items->data)->id)
 		{
 			found = TRUE;
 			break;
@@ -573,7 +576,7 @@ gboolean usr_widgets_set_info (gchar *user)
 	found = FALSE;
 	while (items)
 	{
-		if (info->gid == (guint) atoi ((gchar *) items->data))
+		if (info->gid == ((SystemAccount *) items->data)->id)
 		{
 			found = TRUE;
 			break;
@@ -642,6 +645,7 @@ GtkWidget *usr_init_usermanager_window (GtkWidget *parent_window)
 	return window;
 }
 
+#if FALSE
 gboolean usr_manager_check_prerequisites (void)
 {
 	static gboolean first_time = TRUE;
@@ -725,7 +729,7 @@ gboolean usr_manager_check_prerequisites (void)
 	first_time = FALSE;
 	return TRUE;
 }
-
+#endif
 
 gboolean usr_remove_user (const gchar *user, GError **err)
 {
