@@ -102,10 +102,12 @@ gboolean usr_widgets_set_info (gchar *user);
 gboolean usr_user_exists (const gchar *user);
 void usr_select_user (const gchar *user);
 gboolean usr_select_first_user (void);
-gboolean usr_manager_check_prerequisites (void);
 gboolean usr_remove_user (const gchar *user, GError **err);
 gboolean usr_add_user (const gchar *user, const gchar *passwd, GError **err);
 gboolean usr_save_user_info (GError **err);
 GList *usr_get_available_users (GError **err);
 
+gchar *usr_data_serialize(const PWInfo *info);
+PWInfo *usr_data_deserialize (const gchar *line);
+
 #endif /* __USR_MANAGER_H__ */
