@@ -508,7 +508,7 @@ optmenu_id_changed                     (GtkOptionMenu   *optionmenu,
 		opt_menu_uid = UW("optmenu_uid");
 		opt_menu_gid = UW("optmenu_gid");
 	}
-	if ((gint) optionmenu == (gint) opt_menu_uid)
+	if ((gpointer) optionmenu == (gpointer) opt_menu_uid)
 		gtk_widget_hide (lbl_uid);
 	else
 		gtk_widget_hide (lbl_gid);
